import 'dart:io';

import 'package:args/args.dart';

void main(List<String> arguments) async {
  final parser = ArgParser();

  parser.addFlag(
    'help',
    abbr: 'h',
    help: 'Print this usage information.',
    negatable: false,
  );

  parser.addOption(
    'dir',
    help: 'The directory in which to look for benchmarks.',
    valueHelp: 'String',
    defaultsTo: './benchmark',
  );
  final result = parser.parse(arguments);

  if (result['help']) {
    return print(parser.usage);
  }

  final uri = Uri.parse((result['dir'] ?? './benchmark'));
  Directory directory;
  if (uri.hasAbsolutePath) {
    directory = Directory.fromUri(uri);
  } else {
    directory = Directory(Directory.current.path + '/' + uri.path);
  }

  if (!await directory.exists()) {
    throw Exception('Directory $directory does not exist!');
  }

  final files = directory.listSync(recursive: true)
    ..retainWhere(
      (element) =>
          element.statSync().type == FileSystemEntityType.file &&
          element.path.endsWith('_benchmark.dart'),
    );

  for (final file in files) {
    final result = await Process.run('dart', [file.path]);
    print(result.stdout);
  }
}
