part of benchmark;

class BenchmarkRunner {
  final String name;

  final GroupEntry _group;

  final void Function() benchmark;

  final int iterations;

  final Duration duration;

  BenchmarkRunner(
    this.name,
    this._group,
    this.benchmark, {
    this.iterations = 10,
    this.duration = const Duration(seconds: 2),
  })  : assert(iterations != null),
        assert(duration != null);

  void warmup() => run();

  void exercise() {
    for (var i = 0; i < iterations; i++) {
      run();
    }
  }

  void setUp(GroupEntry forGroup) {
    if (forGroup._parent != null) {
      setUp(forGroup._parent);
    }
    for (final setUp in forGroup._setUps) {
      setUp();
    }
  }

  void setUpEach(GroupEntry forGroup) {
    if (forGroup._parent != null) {
      setUpEach(forGroup._parent);
    }
    for (final setUpEach in forGroup._setUpEachs) {
      setUpEach();
    }
  }

  void tearDown(GroupEntry forGroup) {
    for (final tearDown in forGroup._tearDowns) {
      tearDown();
    }
    if (forGroup._parent != null) {
      tearDown(forGroup._parent);
    }
  }

  void tearDownEach(GroupEntry forGroup) {
    for (final tearDownEach in forGroup._tearDownEachs) {
      tearDownEach();
    }
    if (forGroup._parent != null) {
      tearDownEach(forGroup._parent);
    }
  }

  double measureFor(Function f, int minimumMillis) {
    var minimumMicros = minimumMillis * 1000,
        iter = 0,
        watch = Stopwatch(),
        elapsed = 0;

    while (elapsed < minimumMicros) {
      setUpEach(_group);

      watch.start();
      f();
      watch.stop();

      tearDownEach(_group);

      elapsed = watch.elapsedMicroseconds;
      iter++;
    }
    return iter == 0 ? 0 : elapsed / iter;
  }

  double measure() {
    setUp(_group);

    // Warmup for at least 100ms. Discard result.
    measureFor(warmup, 100);

    // Run the benchmark for at least 2000ms.
    var result = measureFor(exercise, duration.inMilliseconds);

    tearDown(_group);

    return result;
  }

  void run() => benchmark();
}
