part of benchmark;

class GroupEntry {
  final String name;

  final GroupEntry _parent;

  final List<GroupEntry> _groups = [];

  final List<BenchmarkRunner> _benchmarks = [];

  final List<void Function()> _setUps = [];
  final List<void Function()> _setUpEachs = [];
  final List<void Function()> _setUpAlls = [];

  final List<void Function()> _tearDowns = [];
  final List<void Function()> _tearDownEachs = [];
  final List<void Function()> _tearDownAlls = [];

  GroupEntry(this.name, this._parent);

  void benchmark(
    String name,
    void Function() callback, {
    int iterations = 10,
    Duration duration = const Duration(seconds: 2),
  }) {
    final GroupEntry zonedGroup = Zone.current[#test.group] ?? this;
    zonedGroup._benchmarks.add(BenchmarkRunner(
      name,
      zonedGroup,
      callback,
      iterations: iterations,
      duration: duration,
    ));
  }

  void group(String name, void Function() callback) {
    final GroupEntry zonedGroup = Zone.current[#test.group] ?? this;
    final group = GroupEntry(name, zonedGroup);
    runZoned(callback, zoneValues: {#test.group: group});
    zonedGroup._groups.add(group);
  }

  void setUp(void Function() callback) {
    final GroupEntry zonedGroup = Zone.current[#test.group] ?? this;
    zonedGroup._setUps.add(callback);
  }

  void setUpEach(void Function() callback) {
    final GroupEntry zonedGroup = Zone.current[#test.group] ?? this;
    zonedGroup._setUpEachs.add(callback);
  }

  void setUpAll(void Function() callback) {
    final GroupEntry zonedGroup = Zone.current[#test.group] ?? this;
    zonedGroup._setUpAlls.add(callback);
  }

  void tearDown(void Function() callback) {
    final GroupEntry zonedGroup = Zone.current[#test.group] ?? this;
    zonedGroup._tearDowns.add(callback);
  }

  void tearDownEach(void Function() callback) {
    final GroupEntry zonedGroup = Zone.current[#test.group] ?? this;
    zonedGroup._tearDownEachs.add(callback);
  }

  void tearDownAll(void Function() callback) {
    final GroupEntry zonedGroup = Zone.current[#test.group] ?? this;
    zonedGroup._tearDownAlls.add(callback);
  }

  void _setUpAll() {
    for (final setUpAll in _setUpAlls) {
      setUpAll();
    }

    for (final group in _groups) {
      group._setUpAll();
    }
  }

  Map<String, dynamic> _benchmark() {
    final report = <String, dynamic>{};
    for (final benchmark in _benchmarks) {
      report[benchmark.name] = benchmark.measure();
    }
    for (final group in _groups) {
      report[group.name] = group._benchmark();
    }
    return report;
  }

  void _tearDownAll() {
    for (final tearDownAll in _tearDownAlls) {
      tearDownAll();
    }

    for (final group in _groups) {
      group._tearDownAll();
    }
  }

  Map<String, dynamic> _measure() {
    _setUpAll();
    final report = _benchmark();
    _tearDownAll();
    return report;
  }
}
